import ijson
import AlteryxPythonSDK as Sdk

from typing import IO, Union, Callable

from connectors_core.alteryx.helpers import build_record_info
from .interfaces import FileInputReader


class JsonPivotInputReader(FileInputReader):
    NameFieldName = "JSON_Name"
    StringValueFieldName = "JSON_StringValue"
    IntValueFieldName = "JSON_IntegerValue"
    FloatValueFieldName = "JSON_FloatValue"
    BooleanValueFieldName = "JSON_BooleanValue"

    def __init__(
            self,
            stream: IO,
            output_anchor: Sdk.OutputAnchor,
            alteryx_engine: Sdk.AlteryxEngine,
            stream_length_bytes: Union[int, None] = None,
            field_source_resolver: Callable[[Sdk.Field], str] = lambda f: '',
            force_string_values: bool = False,
            max_field_size=255
    ):
        super().__init__(stream, output_anchor, alteryx_engine, stream_length_bytes, field_source_resolver)
        self.force_string_values = force_string_values
        self.max_field_size = max_field_size

    def init(self):
        record_fields = [
            {
                'name': JsonPivotInputReader.NameFieldName,
                'type': Sdk.FieldType.v_wstring,
                'size': self.max_field_size,
                'scale': 0,
                'description': 'JSON Path to the value'
            },
            {
                'name': JsonPivotInputReader.StringValueFieldName,
                'type': Sdk.FieldType.v_wstring,
                'size': self.max_field_size,
                'scale': 0,
                'description': 'JSON string value'
            }
        ]

        if not self.force_string_values:
            record_fields.append(
                {
                    'name': JsonPivotInputReader.IntValueFieldName,
                    'type': Sdk.FieldType.int64,
                    'size': 0,
                    'scale': 0,
                    'description': 'JSON Integer value'
                }
            )
            record_fields.append(
                {
                    'name': JsonPivotInputReader.FloatValueFieldName,
                    'type': Sdk.FieldType.double,
                    'size': 0,
                    'scale': 0,
                    'description': 'JSON Floating Point Number value'
                }
            )
            record_fields.append(
                {
                    'name': JsonPivotInputReader.BooleanValueFieldName,
                    'type': Sdk.FieldType.bool,
                    'size': 0,
                    'scale': 0,
                    'description': 'JSON Boolean value'
                }
            )

        self.record_info = build_record_info(
            self.alteryx_engine,
            record_fields,
            get_field_source=self.field_source_resolver
        )

        self.output_anchor.init(self.record_info)
        self.record_creator = self.record_info.construct_record_creator()
        self.is_initialized = True
        return self.record_info

    def read(self, n_record_limit=-1):
        record_count = 0

        for record in self.parse():
            record_info_out = self.record_info.clone()
            self.record_creator.reset()

            name_idx = record_info_out.get_field_num(JsonPivotInputReader.NameFieldName)
            record_info_out[name_idx].set_from_string(self.record_creator, record[JsonPivotInputReader.NameFieldName])

            string_val_idx = record_info_out.get_field_num(JsonPivotInputReader.StringValueFieldName)
            string_field = record_info_out[string_val_idx]
            if record[JsonPivotInputReader.StringValueFieldName] is not None:
                string_field.set_from_string(
                    self.record_creator, record[JsonPivotInputReader.StringValueFieldName]
                )
            else:
                string_field.set_null(self.record_creator)

            if not self.force_string_values:
                int_val_idx = record_info_out.get_field_num(JsonPivotInputReader.IntValueFieldName)
                int_field = record_info_out[int_val_idx]
                if record[JsonPivotInputReader.IntValueFieldName] is not None:
                    int_field.set_from_int64(
                        self.record_creator, record[JsonPivotInputReader.IntValueFieldName]
                    )
                else:
                    int_field.set_null(self.record_creator)

                float_val_idx = record_info_out.get_field_num(JsonPivotInputReader.FloatValueFieldName)
                float_field = record_info_out[float_val_idx]
                if record[JsonPivotInputReader.FloatValueFieldName] is not None:
                    float_field.set_from_double(
                        self.record_creator, record[JsonPivotInputReader.FloatValueFieldName]
                    )
                else:
                    float_field.set_null(self.record_creator)

                bool_val_idx = record_info_out.get_field_num(JsonPivotInputReader.BooleanValueFieldName)
                bool_field = record_info_out[bool_val_idx]
                if record[JsonPivotInputReader.BooleanValueFieldName] is not None:
                    bool_field.set_from_bool(
                        self.record_creator, record[JsonPivotInputReader.BooleanValueFieldName]
                    )
                else:
                    bool_field.set_null(self.record_creator)

            finalized_record_out = self.record_creator.finalize_record()
            self.output_anchor.push_record(finalized_record_out, no_auto_close=False)

            self.report_input_stream_progress()

            record_count = record_count + 1
            if 0 < n_record_limit <= record_count:
                break

        return record_count

    def parse(self):
        parser = ijson.parse(self.stream)
        for key, event, value in parser:
            if event in ("string", "boolean", "number"):
                yield self.make_record(value, key)

    def make_record(self, value, json_path):
        return {
            self.NameFieldName: json_path,
            self.StringValueFieldName: str(value) if isinstance(value, str) or self.force_string_values else None,
            self.IntValueFieldName: value if isinstance(value, int) and not self.force_string_values else None,
            self.FloatValueFieldName: value if isinstance(value, float) and not self.force_string_values else None,
            self.BooleanValueFieldName: value if isinstance(value, bool) and not self.force_string_values else None
        }
